function m=solver_dyn(m)
tic
%Newmark
b=0.25;
g=0.5;

global DATA
time=DATA.timeline;
dtime=gradient(time);

rep=m.repsubdof;
repud=m.repud;
repvd=m.repvd;

uold=0*m.ubc(:,1);
vold=0*m.ubc(:,1);
aold=0*m.ubc(:,1);

pvd_file('sol',1,numel(time));

for istep=1:numel(time)
    dt=dtime(istep);
    k=m.m/(dt.^2*b)+m.k;
    
    %% RIGHT HAND SIDE
    % external forces
    f=m.fbc(:,istep);

    % prescribed displacement
    f=f-k*m.ubc(:,istep);
    
    % prescribed velocity
    uvd=0*vold;
    uvd(repvd)=uold(repvd) + (1-b/g)*dt*vold(repvd) + dt*m.vbc(repvd,istep)*b/g + dt^2*aold(repvd)*(0.5-b/g);
    f=f-k*uvd;

    % inertial forces
    f=f+m.m*(uold/(dt^2*b) + vold/(dt*b) + (0.5-b)/b*aold);
    
    % Displacement
    u=m.ubc(:,istep);
    u(repvd)=uvd(repvd);
    u(rep)=k(rep,rep)\f(rep);
    
    % Acceleration
    a=(u-uold)/(dt^2*b) - vold/(dt*b) - (0.5-b)/b*aold;

    % Velocity
    v=vold + dt*(1-g)*aold + dt*g*a;

    % post
    m=call_after_solve(m,u);
    vtk_sol(m,1,istep);
    norm(u)
    uold=u;
    vold=v;
    aold=a;
end

m.u=u;

disp(['Dynamic Solver... ' num2str(toc,2) ' s']);

